#!/usr/bin/env python3

import argparse

from Qt.QtWidgets import QApplication
import sys

from .core import MultiGifView


def show_gifs(*filenames, max_columns=2, titles=True, zoom=None):
    """Show gifs in a Qt window

    Any number of gifs can be opened. Each will be in a new column until there are
    max_columns columns. After that new rows will be created and filled until all the
    gifs are shown.

    Parameters
    ----------
    *args - list of str or pathlib.Path
        The .gif files to open
    max_columns : int, default 2
        Maximum number of columns to use
    zoom : float, optional
        Set the initial zoom level of the gifs (in %)
    """
    app = QApplication(sys.argv)
    window = MultiGifView(
        filenames, max_columns=max_columns, titles=titles, initial_zoom=zoom
    )
    window.show()
    window.reset_minimum_size()

    return app.exec_()


def main():
    """Simple gif viewer.

    Commands:
    play - space;
    previous frame - p or left-arrow;
    next frame - n or right-arrow;
    beginning - b or up arrow;
    end - e or down arrow;
    zoom in - +
    zoom out - -
    quit - q, Ctrl-q, Ctrl-w or Ctrl-x.
    """
    # Make sure application exits on Ctrl-C
    import signal

    signal.signal(signal.SIGINT, signal.SIG_DFL)

    # Use argparse to add help
    parser = argparse.ArgumentParser(description=main.__doc__)
    parser.add_argument("file", nargs="+", help=".gif files to open")
    parser.add_argument(
        "-c",
        "--max-columns",
        type=int,
        default=2,
        help="maximum number of columns to use (default 2)",
    )
    parser.add_argument(
        "-n",
        "--no-titles",
        action="store_true",
        default=False,
        help="Disable titles above gifs",
    )
    from multigifview import __version__

    parser.add_argument(
        "-v", "--version", action="version", version="%(prog)s {}".format(__version__)
    )
    parser.add_argument(
        "-z",
        "--zoom",
        type=float,
        default=None,
        help="Initial zoom setting in percent",
    )
    args = parser.parse_args()

    exit_code = show_gifs(
        *args.file,
        max_columns=args.max_columns,
        titles=not args.no_titles,
        zoom=args.zoom
    )

    sys.exit(exit_code)


if __name__ == "__main__":
    main()



from pathlib import Path

from .mainwindow import Ui_MainWindow
from Qt.QtWidgets import (
    QApplication,
    QMainWindow,
    QLabel,
    QShortcut,
    QSizePolicy,
    QSpacerItem,
    QStyle,
    QVBoxLayout,
)
from Qt.QtGui import QMovie, QKeySequence
from Qt.QtCore import QSize
from Qt.QtCore import Qt as QtCoreQt


class MultiGifView(QMainWindow, Ui_MainWindow):
    """A program for viewing .gif files"""

    def __init__(self, filenames, *, max_columns, titles, initial_zoom=None):
        super().__init__(None)
        self.setupUi(self)

        # extra keyboard shortcuts
        quit_shortcut = QShortcut(QKeySequence("Q"), self)
        quit_shortcut.activated.connect(QApplication.instance().quit)
        quit_shortcut = QShortcut(QKeySequence("Ctrl+Q"), self)
        quit_shortcut.activated.connect(QApplication.instance().quit)
        quit_shortcut = QShortcut(QKeySequence("Ctrl+X"), self)
        quit_shortcut.activated.connect(QApplication.instance().quit)
        quit_shortcut = QShortcut(QKeySequence("Ctrl+W"), self)
        quit_shortcut.activated.connect(QApplication.instance().quit)
        previous_shortcut = QShortcut(QKeySequence("left"), self)
        previous_shortcut.activated.connect(self.previous_action)
        next_shortcut = QShortcut(QKeySequence("right"), self)
        next_shortcut.activated.connect(self.next_action)
        beginning_shortcut = QShortcut(QKeySequence("up"), self)
        beginning_shortcut.activated.connect(self.beginning_action)
        end_shortcut = QShortcut(QKeySequence("down"), self)
        end_shortcut.activated.connect(self.end_action)

        def set_clicked(widget, function):
            widget.clicked.connect(function)
            if hasattr(function, "__doc__"):
                widget.setToolTip(function.__doc__.strip())

        set_clicked(self.play_button, self.play_action)
        set_clicked(self.previous_button, self.previous_action)
        set_clicked(self.next_button, self.next_action)
        set_clicked(self.beginning_button, self.beginning_action)
        set_clicked(self.end_button, self.end_action)

        set_clicked(self.quit_button, self.quit_action)

        set_clicked(self.zoom_out_button, self.zoom_out_action)
        set_clicked(self.zoom_in_button, self.zoom_in_action)
        set_clicked(self.zoom_reset_button, self.zoom_reset_action)
        self.zoom_box.editingFinished.connect(self.zoom_changed_action)
        self.zoom_box.setToolTip(self.zoom_changed_action.__doc__.strip())

        self.zoom = 100.0

        if max_columns < 1:
            raise ValueError(f"Number of columns must be positive, got {max_columns}")
        n_columns = min(max_columns, len(filenames))
        self.columns = [self.column0]
        for i in range(n_columns)[1:]:
            # Create column
            column = QVBoxLayout()
            column.setObjectName(f"column{i}")
            spacerItem = QSpacerItem(0, 0, QSizePolicy.Minimum, QSizePolicy.Expanding)
            column.addItem(spacerItem)
            self.gif_layout.addLayout(column)
            self.columns.append(column)

        self.extra_movies = []
        self.extra_gif_widgets = []
        for i, arg in enumerate(filenames):
            gif_widget = QLabel(self.centralwidget)
            gif_widget.setText("")
            gif_widget.setToolTip(arg)
            gif_widget.setObjectName(f"gif_widget{i + 1}")

            filepath = Path(arg)
            movie = QMovie(str(filepath))
            movie.setCacheMode(QMovie.CacheAll)
            gif_widget.setMovie(movie)
            movie.jumpToFrame(0)
            movie.unscaled_width = movie.currentImage().width()
            movie.unscaled_height = movie.currentImage().height()

            column = self.columns[i % n_columns]
            position = column.count() - 1

            if titles:
                title_widget = QLabel(self.centralwidget)
                title_widget.setText(arg)
                title_widget.setAlignment(QtCoreQt.AlignCenter)
                title_widget.setObjectName(f"title_widget{i + 1}")

                column.insertWidget(position, title_widget)
                position = position + 1

            column.insertWidget(position, gif_widget)

            self.extra_movies.append(movie)
            self.extra_gif_widgets.append(gif_widget)

        # want the longest-running gif to be the one that's directly controlled, so that
        # it can play all the way to the end, not have to stop when the first movie
        # reaches its last frame
        frame_counts = [m.frameCount() for m in self.extra_movies]
        ind_longest = frame_counts.index(max(frame_counts))
        self.movie = self.extra_movies.pop(ind_longest)

        # Create actions so extra movies follow self.movie
        self.movie.frameChanged.connect(self.change_frames)

        if initial_zoom is not None:
            self.set_zoom(initial_zoom)

        # Set a sensible initial width
        (
            left_margin,
            top_margin,
            right_margin,
            bottom_margin,
        ) = self.verticalLayout_main.getContentsMargins()
        scrollable_width = (
            left_margin
            + sum([c.sizeHint().width() for c in self.columns])
            + self.columns[0].spacing()
            + sum([2 * c.spacing() for c in self.columns[1:-1]])
            + self.columns[-1].spacing()
            + self.scrollArea.verticalScrollBar().sizeHint().width()
            + right_margin
        )
        max_width = (
            QApplication.desktop().availableGeometry().width()
            - left_margin
            - right_margin
        )
        self.scrollArea.setMinimumWidth(min(scrollable_width, max_width))

        # Set a sensible initial height
        column_heights = [c.sizeHint().height() for c in self.columns]
        highest_col_index = column_heights.index(max(column_heights))
        scrollable_height = (
            top_margin
            + column_heights[highest_col_index]
            + self.columns[highest_col_index].spacing()
            + self.scrollArea.horizontalScrollBar().sizeHint().height()
            + bottom_margin
        )
        control_bar_height = (
            self.horizontalLayout.sizeHint().height()
            + 2 * self.horizontalLayout.spacing()
        )
        menu_bar_height = self.menubar.sizeHint().height()
        title_bar_height = QApplication.style().pixelMetric(QStyle.PM_TitleBarHeight)
        max_height = (
            QApplication.desktop().availableGeometry().height()
            - top_margin
            - bottom_margin
            - control_bar_height
            - menu_bar_height
            - title_bar_height
        )
        self.scrollArea.setMinimumHeight(min(scrollable_height, max_height))

    def play_action(self):
        """<html><head/><body><p><b>Play</b><br>
        space</p></body></html>
        """
        if self.movie.state() == QMovie.Running:
            self.movie.setPaused(True)
        elif self.movie.state() == QMovie.Paused:
            self.movie.setPaused(False)
        else:
            self.movie.start()

    def previous_action(self):
        """<html><head/><body><p><b>Back one frame</b><br>
        p, left</p></body></html>
        """
        self.movie.jumpToFrame(
            (self.movie.currentFrameNumber() - 1) % self.movie.frameCount()
        )

    def next_action(self):
        """<html><head/><body><p><b>Forward one frame</b><br>
        n, right</p></body></html>
        """
        self.movie.jumpToNextFrame()

    def beginning_action(self):
        """<html><head/><body><p><b>Back to beginning</b><br>
        b, up</p></body></html>
        """
        self.movie.jumpToFrame(0)

    def end_action(self):
        """<html><head/><body><p><b>Forward to end</b><br>
        e, down</p></body></html>
        """
        self.movie.jumpToFrame(self.movie.frameCount() - 1)

    def quit_action(self):
        """<html><head/><body><p><b>Quit</b><br>
        q, Ctrl-q, Ctrl-w, Ctrl-x</p></body></html>
        """
        QApplication.instance().quit

    def change_frames(self, new_frame):
        """Change all the frames in step"""
        for movie in self.extra_movies:
            length = movie.frameCount()
            if new_frame < length:
                movie.jumpToFrame(new_frame)
            else:
                movie.jumpToFrame(length - 1)

    def reset_minimum_size(self):
        """Allow window to be shrunk from default size"""
        self.scrollArea.setMinimumWidth(0)
        self.scrollArea.setMinimumHeight(0)

    def set_zoom(self, new_zoom):
        try:
            self.zoom = float(new_zoom)
        except ValueError:
            # Make the box red
            self.zoom_box.setStyleSheet("QLineEdit { background-color: #aa0000 }")
            return
        self.zoom_box.setStyleSheet("QLineEdit { background-color: #FFFFFF }")
        self.zoom_box.setText(str(int(round(self.zoom))))

        current_frame = self.movie.currentFrameNumber()
        scale_factor = self.zoom / 100.0

        self.movie.setScaledSize(
            QSize(
                scale_factor * self.movie.unscaled_width,
                scale_factor * self.movie.unscaled_height,
            )
        )
        for movie in self.extra_movies:
            movie.setScaledSize(
                QSize(
                    scale_factor * movie.unscaled_width,
                    scale_factor * movie.unscaled_height,
                )
            )

        # change frame to make the image re-draw
        self.movie.jumpToFrame(0 if current_frame > 0 else 1)
        self.movie.jumpToFrame(current_frame)

    def zoom_changed_action(self):
        """<html><head/><body><p><b>Set zoom factor</b></p></body></html>"""
        self.set_zoom(self.zoom_box.text())

    def zoom_out_action(self):
        """<html><head/><body><p><b>Zoom out</b><br>
        -</p></body></html>
        """
        new_zoom = 0.8 * self.zoom
        if self.zoom >= 100.0:
            # Round to nearest 10%
            new_zoom = round(new_zoom, -1)
        elif self.zoom > 20.0:
            # Round to nearest 5%
            new_zoom = round(2.0 * new_zoom, -1) / 2.0
        elif self.zoom > 5.0:
            # Round to nearest 1%
            new_zoom = round(new_zoom)
        elif self.zoom > 0.5:
            # Round to nearest 0.1%
            new_zoom = round(new_zoom, 1)
        elif self.zoom > 0.05:
            # Round to nearest 0.01%
            new_zoom = round(new_zoom, 2)

        self.set_zoom(new_zoom)

    def zoom_in_action(self):
        """<html><head/><body><p><b>Zoom in</b><br>
        +</p></body></html>
        """
        new_zoom = 1.25 * self.zoom
        if self.zoom >= 100.0:
            # Round to nearest 10%
            new_zoom = round(new_zoom, -1)
        elif self.zoom > 20.0:
            # Round to nearest 5%
            new_zoom = round(2.0 * new_zoom, -1) / 2.0
        elif self.zoom > 5.0:
            # Round to nearest 1%
            new_zoom = round(new_zoom)
        elif self.zoom > 0.5:
            # Round to nearest 0.1%
            new_zoom = round(new_zoom, 1)
        elif self.zoom > 0.05:
            # Round to nearest 0.01%
            new_zoom = round(new_zoom, 2)

        self.set_zoom(new_zoom)

    def zoom_reset_action(self):
        """<html><head/><body><p><b>Reset original size</b><br>
        =</p></body></html>
        """
        self.set_zoom(100.0)



from .__main__ import main, show_gifs

try:
    from importlib.metadata import version, PackageNotFoundError
except ModuleNotFoundError:
    from importlib_metadata import version, PackageNotFoundError
try:
    __version__ = version(__name__)
except PackageNotFoundError:
    # package is not installed
    pass



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'mainwindow.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(516, 151)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.centralwidget.sizePolicy().hasHeightForWidth())
        self.centralwidget.setSizePolicy(sizePolicy)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_main = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_main.setObjectName("verticalLayout_main")
        self.scrollArea = QtWidgets.QScrollArea(self.centralwidget)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 496, 73))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gif_layout = QtWidgets.QHBoxLayout(self.scrollAreaWidgetContents)
        self.gif_layout.setObjectName("gif_layout")
        self.column0 = QtWidgets.QVBoxLayout()
        self.column0.setObjectName("column0")
        spacerItem = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.column0.addItem(spacerItem)
        self.gif_layout.addLayout(self.column0)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.verticalLayout_main.addWidget(self.scrollArea)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setSizeConstraint(QtWidgets.QLayout.SetFixedSize)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.beginning_button = QtWidgets.QPushButton(self.centralwidget)
        self.beginning_button.setText("")
        icon = QtGui.QIcon.fromTheme("media-skip-backward")
        self.beginning_button.setIcon(icon)
        self.beginning_button.setObjectName("beginning_button")
        self.horizontalLayout.addWidget(self.beginning_button)
        self.previous_button = QtWidgets.QPushButton(self.centralwidget)
        self.previous_button.setText("")
        icon = QtGui.QIcon.fromTheme("media-seek-backward")
        self.previous_button.setIcon(icon)
        self.previous_button.setObjectName("previous_button")
        self.horizontalLayout.addWidget(self.previous_button)
        self.play_button = QtWidgets.QPushButton(self.centralwidget)
        self.play_button.setText("")
        icon = QtGui.QIcon.fromTheme("media-playback-start")
        self.play_button.setIcon(icon)
        self.play_button.setObjectName("play_button")
        self.horizontalLayout.addWidget(self.play_button)
        self.next_button = QtWidgets.QPushButton(self.centralwidget)
        self.next_button.setText("")
        icon = QtGui.QIcon.fromTheme("media-seek-forward")
        self.next_button.setIcon(icon)
        self.next_button.setObjectName("next_button")
        self.horizontalLayout.addWidget(self.next_button)
        self.end_button = QtWidgets.QPushButton(self.centralwidget)
        self.end_button.setText("")
        icon = QtGui.QIcon.fromTheme("media-skip-forward")
        self.end_button.setIcon(icon)
        self.end_button.setObjectName("end_button")
        self.horizontalLayout.addWidget(self.end_button)
        self.line = QtWidgets.QFrame(self.centralwidget)
        self.line.setFrameShape(QtWidgets.QFrame.VLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.horizontalLayout.addWidget(self.line)
        self.zoom_out_button = QtWidgets.QPushButton(self.centralwidget)
        self.zoom_out_button.setText("")
        icon = QtGui.QIcon.fromTheme("zoom-out")
        self.zoom_out_button.setIcon(icon)
        self.zoom_out_button.setObjectName("zoom_out_button")
        self.horizontalLayout.addWidget(self.zoom_out_button)
        self.zoom_box = QtWidgets.QLineEdit(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.zoom_box.sizePolicy().hasHeightForWidth())
        self.zoom_box.setSizePolicy(sizePolicy)
        self.zoom_box.setMinimumSize(QtCore.QSize(33, 26))
        self.zoom_box.setMaximumSize(QtCore.QSize(33, 26))
        self.zoom_box.setObjectName("zoom_box")
        self.horizontalLayout.addWidget(self.zoom_box)
        self.zoom_in_button = QtWidgets.QPushButton(self.centralwidget)
        self.zoom_in_button.setToolTip("")
        self.zoom_in_button.setText("")
        icon = QtGui.QIcon.fromTheme("zoom-in")
        self.zoom_in_button.setIcon(icon)
        self.zoom_in_button.setObjectName("zoom_in_button")
        self.horizontalLayout.addWidget(self.zoom_in_button)
        self.zoom_reset_button = QtWidgets.QPushButton(self.centralwidget)
        self.zoom_reset_button.setText("")
        icon = QtGui.QIcon.fromTheme("zoom-original")
        self.zoom_reset_button.setIcon(icon)
        self.zoom_reset_button.setObjectName("zoom_reset_button")
        self.horizontalLayout.addWidget(self.zoom_reset_button)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.quit_button = QtWidgets.QPushButton(self.centralwidget)
        self.quit_button.setText("")
        icon = QtGui.QIcon.fromTheme("window-close")
        self.quit_button.setIcon(icon)
        self.quit_button.setObjectName("quit_button")
        self.horizontalLayout.addWidget(self.quit_button)
        self.verticalLayout_main.addLayout(self.horizontalLayout)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 516, 22))
        self.menubar.setObjectName("menubar")
        MainWindow.setMenuBar(self.menubar)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "multigifview"))
        self.beginning_button.setShortcut(_translate("MainWindow", "B"))
        self.previous_button.setShortcut(_translate("MainWindow", "P"))
        self.play_button.setShortcut(_translate("MainWindow", "Space"))
        self.next_button.setShortcut(_translate("MainWindow", "N"))
        self.end_button.setShortcut(_translate("MainWindow", "E"))
        self.zoom_out_button.setShortcut(_translate("MainWindow", "-"))
        self.zoom_box.setText(_translate("MainWindow", "100"))
        self.zoom_in_button.setShortcut(_translate("MainWindow", "+"))
        self.zoom_reset_button.setShortcut(_translate("MainWindow", "="))
